import os
import json
import logging
from datetime import date, time, timedelta

log = logging.getLogger(__name__)


class FitbitDatastore(object):
    '''
    Local data store of Fitbit json objects.
    '''

    def __init__(self, root):
        self.root = os.path.abspath(root)

    def filename(self, *args):
        return os.path.join(self.root, *args)

    @staticmethod
    def write(filename, data):
        dirname = os.path.dirname(filename)
        os.makedirs(dirname, exist_ok=True)
        with open(filename, 'w') as f:
            f.write(json.dumps(data, indent=2, sort_keys=True))

    @staticmethod
    def read(filename):
        return json.load(open(filename))

    def day_filenames(self, name):
        # TODO use profile['memberSince']
        start = date(2019, 9, 18)
        days = 0
        while 1:
            day = start + timedelta(days=days)
            days += 1
            if day > date.today():
                return

            filename = self.filename(
                name,
                '{:04d}'.format(day.year),
                '{}.{:04d}.{:02d}.{:02d}.json'.format(name, day.year, day.month, day.day),
            )
            yield day, filename

    def get_proile(self):
        return self.read(self.filename('profile.json'))

    def get_sleep(self):
        '''
        Return sleep data from the local store.
        Returns: [{sleep_data}, ...]
        where `sleep_data` is the inner dict from
        https://dev.fitbit.com/build/reference/web-api/sleep/
        '''
        for dir, dirs, files in os.walk(self.filename('sleep')):
            for file in files:
                filename = os.path.join(dir, file)
                data = self.read(filename)
                yield from data

    def get_activities(self):
        '''
        Return activities data from the local store.
        Yeilds: (day, {activities_data})
        where `activities_data` is the inner dict from
        https://dev.fitbit.com/build/reference/web-api/activity/
        '''
        for day, filename in self.day_filenames('activities'):
            if not os.path.isfile(filename):
                continue
            data = self.read(filename)
            yield day, data

    def get_heartrate_intraday(self):
        '''
        Return heartrate intraday data from the local store.
        Returns: [{hr_data}, ...]
        where `hr_data` is:
        {
            "date": "2016-07-08",
            "minutes": [int, ...]
        }
        minutes is an array of 1440 minutes in the day and the HR during that minute
        '''

        def compress(data):
            minutes = [None] * 24 * 60
            for o in data:
                h, m, s = map(int, o['time'].split(':'))
                i = h * 60 + m
                minutes[i] = o['value']
            return minutes

        for d, filename in self.day_filenames('heartrate_intraday'):
            if not os.path.isfile(filename):
                continue
            data = self.read(filename)
            if not data:
                continue
            yield {'date': d.isoformat(), 'minutes': compress(data)}



import re
import os
import json
import logging
import time
import urllib.parse
import webbrowser

import requests


__all__ = ['FitbitClient']


log = logging.getLogger(__name__)


class FitbitClient(object):
    def __init__(self, access_token):
        self.access_token = access_token
        self.session = requests.Session()
        self.session.timeout = 30
        self.session.headers['Authorization'] = 'Bearer ' + access_token
        self.profile = self.get_profile()

    @property
    def user_id(self):
        return self.profile['encodedId']

    def get_profile(self):
        r = self.session.get('https://api.fitbit.com/1/user/-/profile.json')
        r.raise_for_status()
        return json.loads(r.text)['user']

    def get_sleep(self, date):
        r = self.session.get('https://api.fitbit.com/1.2/user/{}/sleep/date/{}.json'
            .format(self.user_id, str(date)))
        r.raise_for_status()
        return json.loads(r.text)['sleep']

    def get_sleep_range(self, date_start, date_end):
        r = self.session.get('https://api.fitbit.com/1.2/user/{}/sleep/date/{}/{}.json'
            .format(self.user_id, str(date_start), str(date_end)))
        r.raise_for_status()
        return json.loads(r.text)['sleep']

    def get_heartrate_range(self, date_start, date_end):
        r = self.session.get('https://api.fitbit.com/1/user/-/activities/heart/date/{}/{}.json'
            .format(str(date_start), str(date_end)))
        r.raise_for_status()
        return json.loads(r.text)['activities-heart']

    def get_heartrate_intraday(self, date):
        r = self.session.get('https://api.fitbit.com/1/user/-/activities/heart/date/{}/{}/1min.json'
            .format(str(date), str(date)))
        r.raise_for_status()
        return json.loads(r.text)['activities-heart-intraday']['dataset']

    def get_activities(self, date):
        r = self.session.get('https://api.fitbit.com/1/user/-/activities/date/{}.json'
            .format(str(date)))
        r.raise_for_status()
        return json.loads(r.text)

    def get_steps(self):
        raise NotImplementedError



import argparse
import configparser
import getpass
import logging
import json
import just
import webbrowser
import getpass

import requests

from nostalgia_fitbit import Fitbit, FitbitAuth
from nostalgia_fitbit.export import FitbitExport

logging.basicConfig(level=logging.DEBUG)


def main():
    conf_path = "~/nostalgia_data/config/fitbit/config.json"
    if not just.exists(conf_path):
        webbrowser.open("https://dev.fitbit.com/apps/new")
        webbrowser.open(
            "https://raw.githubusercontent.com/nostalgia-dev/nostalgia_fitbit/master/docs/fitbit_app.png"
        )
        client_id = getpass.getpass("Client ID: ")
        client_secret = getpass.getpass("Client Secret: ")
        info = {"client_id": client_id, "client_secret": client_secret}
        just.write(info, conf_path)
        print("Saved in:", conf_path)
    config = just.read(conf_path)
    if not config["client_id"] or not config["client_secret"]:
        msg = "Fill in a value for client_id and client_secret in '{}'".format(conf_path)
        raise ValueError(msg)

    fa = FitbitAuth(client_id=config['client_id'], client_secret=config['client_secret'])
    fa.ensure_access_token()

    try:
        f = Fitbit(access_token=fa.access_token['access_token'])
        print(json.dumps(f.profile, indent=2))
    except requests.exceptions.HTTPError as e:
        print(e.response.status_code)
        if e.response.status_code == 429:
            print(e.response.headers)
            return
        raise

    export = FitbitExport(f, profile=f.profile)

    export.sync_sleep()
    export.sync_heartrate_intraday()


if __name__ == '__main__':
    main()



'''
Syncs a FitbitDatastore with the most recent data with a FitbitClient
'''
import os
import json
import logging
from datetime import date, time, timedelta

log = logging.getLogger(__name__)


class FitbitSync(object):
    def __init__(self, datastore, client):
        self.datastore = datastore
        self.client = client

    def sync(self):
        self.sync_profile()
        self.sync_sleep()
        self.sync_heartrate()
        self.sync_heartrate_intraday()
        self.sync_activities()

    def sync_profile(self):
        self.datastore.write(self.datastore.filename('profile.json'), self.client.profile)

    def sync_ranged_data(self, name, client_fn):
        '''
        Downloads date-range time series data from
        the FitBit API to the local data store, one month at a time
        '''
        fn = self.datastore.filename
        month = 2015 * 12  # TODO use profile['memberSince']
        while 1:
            date_start = date(month // 12, month % 12 + 1, 1)
            month += 1
            date_end = date(month // 12, month % 12 + 1, 1)

            if date_start > date.today():
                break

            partial = date_end > date.today()
            partial_filename = fn(
                name,
                '{}.{:04d}.{:02d}.partial.json'.format(name, date_start.year, date_start.month),
            )
            filename = fn(
                name, '{}.{:04d}.{:02}.json'.format(name, date_start.year, date_start.month)
            )

            if os.path.isfile(partial_filename):
                os.remove(partial_filename)

            if partial:
                filename = partial_filename
            elif os.path.isfile(filename):
                log.info('Cached: %s', filename)
                continue

            log.info('Downloading: %s', filename)
            data = client_fn(date_start, date_end - timedelta(days=1))
            self.datastore.write(filename, data)

    def sync_sleep(self):
        self.sync_ranged_data('sleep', self.client.get_sleep_range)

    def sync_heartrate(self):
        self.sync_ranged_data('heartrate', self.client.get_heartrate_range)

    def sync_intraday_data(self, name, client_fn):
        '''
        Downloads intraday data from the FitBit API
        to the local data store, one day at a time
        '''
        for day, filename in self.datastore.day_filenames(name):
            if os.path.isfile(filename):
                log.debug('Cached: %s', filename)
                continue

            log.info('Downloading: %s', filename)
            data = client_fn(day)
            self.datastore.write(filename, data)

    def sync_heartrate_intraday(self):
        self.sync_intraday_data('heartrate_intraday', self.client.get_heartrate_intraday)

    def sync_activities(self):
        self.sync_intraday_data('activities', self.client.get_activities)



""" File unrelated to the package, except for convenience in deploying """
import re
import sh
import os

os.system("rm -rf dist/")
os.system("python setup.py sdist bdist_wheel")
os.system("twine upload dist/*")



import json
import pkg_resources
import sys

import dominate
from dominate.tags import *
from dominate.util import *


def read_resource(name):
    return pkg_resources.resource_string('nostalgia_fitbit', name).decode('utf-8')


def make_report(data):
    doc = dominate.document(title='Fitbit Report')

    with doc.head:
        style(raw(read_resource('static/report.css')))
        script(src='http://dominate.js.zkpq.ca/dominate.min.js')
        script(
            src="https://cdnjs.cloudflare.com/ajax/libs/d3/4.12.0/d3.js",
            integrity="sha256-0Lzb1mm7+96oAeDnxAPpfdRdi6jLYTV9XTVt4p6kPg0=",
            crossorigin="anonymous",
        )
        script(raw('\nvar data = '), raw(json.dumps(data)), raw(';\n'))

    with doc.body:
        div('Sleep')
        div(id='sleep')

        div('Heart Rate')
        div(id='heartrate')

        script(raw(read_resource('static/chart.js')))

    return doc.render()


def main(user_id):
    from nostalgia_fitbit import export

    ex = export.FitbitExport(user_id=user_id)
    data = {'sleep': ex.get_sleep(), 'heartrate': ex.get_heartrate_intraday()}
    html = make_report(data)
    with open('report.html', 'w') as f:
        f.write(html)
    print('Wrote report.html', file=sys.stderr)


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('--user', required=True)
    args = parser.parse_args()
    main(args.user)



import os
import time
import json
import just
import logging
from datetime import date, timedelta, datetime
from dateutil.relativedelta import relativedelta

log = logging.getLogger(__name__)


class FitbitExport(object):
    def __init__(self, client=None, user_id=None, profile=None):
        self.root = just.make_path("~/nostalgia_data/input/fitbit")
        self.client = client
        self.user_id = user_id
        self.profile = profile
        if profile is not None:
            self.member_since = datetime.strptime(self.profile["memberSince"], "%Y-%m-%d").date()
        else:
            self.member_since = None

    def filename(self, *args):
        u = self.client and self.client.user_id or self.user_id
        return os.path.join(self.root, u, *args)

    @staticmethod
    def write(filename, data):
        dirname = os.path.dirname(filename)
        os.makedirs(dirname, exist_ok=True)
        with open(filename, 'w') as f:
            f.write(json.dumps(data, indent=2, sort_keys=True))

    def sync_sleep(self):
        date_start = self.member_since.replace(day=1)
        while 1:
            date_end = date_start + relativedelta(months=1)

            if date_start > date.today():
                break

            partial = date_end > date.today()
            partial_filename = self.filename(
                'sleep',
                'sleep.{:04d}.{:02d}.partial.json'.format(date_start.year, date_start.month),
            )
            filename = self.filename(
                'sleep', 'sleep.{:04d}.{:02}.json'.format(date_start.year, date_start.month)
            )

            if os.path.isfile(partial_filename):
                os.remove(partial_filename)

            if partial:
                filename = partial_filename
            if not os.path.isfile(filename):
                log.info('Downloading: %s', filename)
                sleep = self.client.get_sleep_range(date_start, date_end - timedelta(days=1))
                self.write(filename, sleep)
            else:
                log.info('Cached: %s', filename)
            date_start = date_end

    def heartrate_intraday_filenames(self):
        start = self.member_since
        days = 0
        partials = self.filename("heartrate_intraday", "*/*.partial*.json")
        print("partials", partials)
        print("h", just.remove(partials, allow_recursive=True))
        while 1:
            d = start + timedelta(days=days)
            days += 1
            if d > date.today():
                return
            partial = ".partial_{}".format(time.time()) if d == date.today() else ""
            filename = self.filename(
                'heartrate_intraday',
                '{:04d}'.format(d.year),
                'heartrate_intraday.{:04d}.{:02d}.{:02d}{}.json'.format(
                    d.year, d.month, d.day, partial
                ),
            )
            yield d, filename

    def sync_heartrate_intraday(self):
        for d, filename in self.heartrate_intraday_filenames():
            if os.path.isfile(filename):
                log.info('Cached: %s', filename)
                continue

            log.info('Downloading: %s', filename)
            hr = self.client.get_heartrate_intraday(d)
            self.write(filename, hr)

    def get_sleep(self):
        sleep = []
        for dir, dirs, files in os.walk(self.filename('sleep')):
            for file in files:
                filename = os.path.join(dir, file)
                data = json.load(open(filename))
                if not data:
                    continue
                sleep.extend(data)
        return sleep

    def get_heartrate_intraday(self):
        def compress(data):
            minutes = [None] * 24 * 60
            for o in data:
                h, m, s = map(int, o['time'].split(':'))
                i = h * 60 + m
                minutes[i] = o['value']
            return minutes

        heartrate = []
        for d, filename in self.heartrate_intraday_filenames():
            if not os.path.isfile(filename):
                continue
            data = json.load(open(filename))
            if not data:
                continue
            heartrate.append({'date': d.isoformat(), 'minutes': compress(data)})
        return heartrate



# pylint: disable=bad-whitespace

from setuptools import setup

import imp

_version = imp.load_source("nostalgia_fitbit._version", "nostalgia_fitbit/_version.py")

long_description = open('README.md').read()

setup(
    name='nostalgia_fitbit',
    version=_version.__version__,
    author='Pascal van Kooten',
    author_email='pascal@vks.ai',
    license='LICENSE.txt',
    url='http://github.com/nostalgia-dev/nostalgia_fitbit/',
    description='export fitbit data',
    long_description=long_description,
    keywords='fitbit data export json',
    entry_points={'console_scripts': ['nostalgia_fitbit = nostalgia_fitbit.__main__:main']},
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Healthcare Industry',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering :: Medical Science Apps.',
        'Topic :: Scientific/Engineering :: Visualization',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Archiving',
        'Topic :: Utilities',
    ],
    packages=['nostalgia_fitbit'],
    include_package_data=True,
    install_requires=['requests', 'dominate', 'just', 'python-dateutil'],
)



import base64
import http.server
import json
import logging
import os
import time
import webbrowser

import requests


__all__ = ['FitbitAuth']

log = logging.getLogger(__name__)


class RedirectServer(object):
    IP = '127.0.0.1'
    PORT = 18189
    URL = 'http://localhost:{}/auth_code'.format(PORT)

    def __init__(self):
        self.result = None

        class HTTPHandler(http.server.BaseHTTPRequestHandler):
            def do_GET(handler):
                url = urllib.parse.urlparse(handler.path)
                if url.path != '/auth_code':
                    # browser may want favicon.ico, etc
                    handler.send_response(404)
                    handler.end_headers()
                    handler.wfile(b'')
                query = urllib.parse.parse_qs(url.query)
                self.result = query
                handler.send_response(200)
                handler.send_header('Content-type', 'text/html')
                handler.end_headers()
                handler.wfile.write(b'OK<script>window.close();</script>')

        self.handler_class = HTTPHandler

    def get_result(self):
        httpd = http.server.HTTPServer(('127.0.0.1', self.PORT), self.handler_class)
        httpd.timeout = 5
        for i in range(5):
            httpd.handle_request()
            if self.result:
                return self.result
        raise RuntimeError('Failed to auth with browser')


class FitbitAuth(object):
    def __init__(self, client_id, client_secret, access_token_file=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.access_token_file = access_token_file
        self.access_token = None

    def get_auth_code(self):
        log.info('Getting new auth code')
        url = 'https://www.fitbit.com/oauth2/authorize?' + '&'.join(
            '{}={}'.format(k, v)
            for k, v in {
                'response_type': 'code',
                'client_id': self.client_id,
                'redirect_uri': RedirectServer.URL,
                'scope': '%20'.join(
                    (
                        'activity',
                        'heartrate',
                        'location',
                        'nutrition',
                        'profile',
                        'settings',
                        'sleep',
                        'social',
                        'weight',
                    )
                ),
                'expires_in': '31536000',
            }.items()
        )

        redirect = RedirectServer()
        webbrowser.open_new(url)
        result = redirect.get_result()
        return result['code'][0]

    def get_access_token(self):
        log.info('Getting new access token')
        auth_code = self.get_auth_code()
        auth_string = base64.b64encode(
            self.client_id.encode('ascii') + b':' + self.client_secret.encode('ascii')
        ).decode('ascii')
        r = requests.post(
            'https://api.fitbit.com/oauth2/token',
            headers={'Authorization': 'Basic ' + auth_string},
            data={
                'clientId': self.client_id,
                'code': auth_code,
                'grant_type': 'authorization_code',
                'redirect_uri': RedirectServer.URL,
            },
            timeout=30,
        )
        r.raise_for_status()
        return json.loads(r.text)

    def get_access_token_refresh(self, refresh_token):
        log.info('Refreshing access token')
        auth_string = base64.b64encode(
            self.client_id.encode('ascii') + b':' + self.client_secret.encode('ascii')
        ).decode('ascii')
        r = requests.post(
            'https://api.fitbit.com/oauth2/token',
            headers={'Authorization': 'Basic ' + auth_string},
            data={'grant_type': 'refresh_token', 'refresh_token': refresh_token},
            timeout=30,
        )
        r.raise_for_status()
        return json.loads(r.text)

    def ensure_access_token(self):
        access_token = self.access_token
        if not access_token:
            if os.path.isfile(self.access_token_file):
                access_token = json.load(open(self.access_token_file))
            else:
                access_token = self.get_access_token()

        now = int(time.time())
        if now > access_token['time'] + access_token['expires_in'] - 30:
            log.info('Access token is expired')
            access_token = self.get_access_token_refresh(access_token['refresh_token'])

        if self.access_token is not access_token:
            access_token['time'] = now
            with open(self.access_token_file, 'w') as f:
                json.dump(access_token, f, sort_keys=True, indent=2)
            self.access_token = access_token

        return access_token['access_token']
